#ifndef LWIPOPTS_H
#define LWIPOPTS_H
/*
 * enable SYS_LIGHTWEIGHT_PROT in lwipopts.h if you want inter-task protection
 * for certain critical regions during buffer allocation, deallocation and memory
 * allocation and deallocation.
 */
#define SYS_LIGHTWEIGHT_PROT            0

#define NO_SYS                          1

/* ---------- Memory options ---------- */
/* MEM_ALIGNMENT: should be set to the alignment of the CPU for which
   lwIP is compiled. 4 byte alignment -> define MEM_ALIGNMENT to 4, 2
   byte alignment -> define MEM_ALIGNMENT to 2. */

#define MEM_ALIGNMENT                   4

/* MEM_SIZE: the size of the heap memory. If the application will send
a lot of data that needs to be copied, this should be set high. */
#define MEM_SIZE                        1600

#define MEMP_SANITY_CHECK       0

/* MEMP_NUM_PBUF: the number of memp struct pbufs. If the application
   sends a lot of data out of ROM (or other static memory), this
   should be set high. */
#define MEMP_NUM_PBUF                   16

/* Number of raw connection PCBs */
#define MEMP_NUM_RAW_PCB                4

/* MEMP_NUM_UDP_PCB: the number of UDP protocol control blocks. One
   per active UDP "connection". */
#define MEMP_NUM_UDP_PCB                4

/* MEMP_NUM_TCP_PCB: the number of simulatenously active TCP connections. */
#define MEMP_NUM_TCP_PCB                5

/* MEMP_NUM_TCP_PCB_LISTEN: the number of listening TCP connections. */
#define MEMP_NUM_TCP_PCB_LISTEN         8

/* MEMP_NUM_TCP_SEG: the number of simultaneously queued TCP segments. */
#define MEMP_NUM_TCP_SEG                16

/* MEMP_NUM_SYS_TIMEOUT: the number of simulateously active timeouts. */
#define MEMP_NUM_SYS_TIMEOUT            3

/* The following four are used only with the sequential API and can be
   set to 0 if the application only will use the raw API. */
/* MEMP_NUM_NETBUF: the number of struct netbufs. */
#define MEMP_NUM_NETBUF                 2

/* MEMP_NUM_NETCONN: the number of struct netconns. */
#define MEMP_NUM_NETCONN                4

/* MEMP_NUM_APIMSG: the number of struct api_msg, used for
   communication between the TCP/IP stack and the sequential
   programs. */
#define MEMP_NUM_API_MSG                8

/* MEMP_NUM_TCPIPMSG: the number of struct tcpip_msg, which is used
   for sequential API communication and incoming packets. Used in
   src/api/tcpip.c. */
#define MEMP_NUM_TCPIP_MSG              8

/* ---------- Pbuf options ---------- */
/* PBUF_POOL_SIZE: the number of buffers in the pbuf pool. */

#define PBUF_POOL_SIZE                  16

/* PBUF_POOL_BUFSIZE: the size of each pbuf in the pbuf pool. */

#define PBUF_POOL_BUFSIZE               128

/* PBUF_LINK_HLEN: the number of bytes that should be allocated for a
   link level header. Defaults to 14 for Ethernet. */

#define PBUF_LINK_HLEN                  14

/* ---------- ARP options ---------- */

/** Number of active hardware address, IP address pairs cached */
#define ARP_TABLE_SIZE                  10

/**
 * If enabled, outgoing packets are queued during hardware address
 * resolution.
 *
 * This feature has not stabilized yet. Single-packet queueing is
 * believed to be stable, multi-packet queueing is believed to
 * clash with the TCP segment queueing.
 * 
 * As multi-packet-queueing is currently disabled, enabling this
 * _should_ work, but we need your testing feedback on lwip-users.
 *
 */
#define ARP_QUEUEING                    1

/* ---------- IP options ---------- */
/* Define IP_FORWARD to 1 if you wish to have the ability to forward
   IP packets across network interfaces. If you are going to run lwIP
   on a device with only one network interface, define this to 0. */
#define IP_FORWARD                      0

/* If defined to 1, IP options are allowed (but not parsed). If
   defined to 0, all packets with IP options are dropped. */
#define IP_OPTIONS                      1

/** IP reassembly and segmentation. Even if they both deal with IP
 *  fragments, note that these are orthogonal, one dealing with incoming
 *  packets, the other with outgoing packets
 */

/** Reassemble incoming fragmented IP packets */
#define IP_REASSEMBLY                   1

/** Fragment outgoing IP packets if their size exceeds MTU */
#define IP_FRAG                         1

/* ---------- ICMP options ---------- */

#define ICMP_TTL                        255

/* ---------- RAW options ---------- */

#define LWIP_RAW                        1

#define RAW_TTL                        255

/* ---------- DHCP options ---------- */

#define LWIP_DHCP                       0

/* 1 if you want to do an ARP check on the offered address
   (recommended). */
#define DHCP_DOES_ARP_CHECK             0

/* ---------- SNMP options ---------- */
/** @todo SNMP isn't functional yet. */
#define LWIP_SNMP                       0

#define SNMP_PRIVATE_MIB                0

/* ---------- UDP options ---------- */
#define LWIP_UDP                        1

#define UDP_TTL                         255

/* ---------- TCP options ---------- */
#define LWIP_TCP                        1

#define TCP_TTL                         255

#define TCP_WND                         2048

#define TCP_MAXRTX                      12

#define TCP_SYNMAXRTX                   6


/* Controls if TCP should queue segments that arrive out of
   order. Define to 0 if your device is low on memory. */
#define TCP_QUEUE_OOSEQ                 1

/* TCP Maximum segment size. */
#define TCP_MSS                         128 /* A *very* conservative default. */

/* TCP sender buffer space (bytes). */
#define TCP_SND_BUF                     256

/* TCP sender buffer space (pbufs). This must be at least = 2 *
   TCP_SND_BUF/TCP_MSS for things to work. */
#define TCP_SND_QUEUELEN                4 * TCP_SND_BUF/TCP_MSS


/* Maximum number of retransmissions of data segments. */

/* Maximum number of retransmissions of SYN segments. */

/* TCP writable space (bytes). This must be less than or equal
   to TCP_SND_BUF. It is the amount of space which must be
   available in the tcp snd_buf for select to return writable */
#define TCP_SNDLOWAT                    TCP_SND_BUF/2

/* Support loop interface (127.0.0.1) */
#define LWIP_HAVE_LOOPIF                1

#define LWIP_COMPAT_SOCKETS             1

#define TCPIP_THREAD_PRIO               1

#define SLIPIF_THREAD_PRIO              1

#define PPP_THREAD_PRIO                 1

#define DEFAULT_THREAD_PRIO             1


/* ---------- Socket Options ---------- */
/* Enable SO_REUSEADDR and SO_REUSEPORT options */ 


/* ---------- Statistics options ---------- */
#define LWIP_STATS                      1

#if LWIP_STATS

#define LWIP_STATS_DISPLAY 1
#define LINK_STATS  1
#define IP_STATS    1
#define IPFRAG_STATS    1
#define ICMP_STATS  1
#define UDP_STATS   1
#define TCP_STATS   1
#define MEM_STATS   1
#define MEMP_STATS  1
#define PBUF_STATS  1
#define SYS_STATS   1
#define RAW_STATS   0
#endif /* LWIP_STATS */

/* ---------- PPP options ---------- */

#define PPP_SUPPORT                     0      /* Set for PPP */

#if PPP_SUPPORT 

#define NUM_PPP                         1      /* Max PPP sessions. */

#ifndef PAP_SUPPORT
#define PAP_SUPPORT                     0      /* Set for PAP. */
#endif

#ifndef CHAP_SUPPORT
#define CHAP_SUPPORT                    0      /* Set for CHAP. */
#endif

#define MSCHAP_SUPPORT                  0      /* Set for MSCHAP (NOT FUNCTIONAL!) */
#define CBCP_SUPPORT                    0      /* Set for CBCP (NOT FUNCTIONAL!) */
#define CCP_SUPPORT                     0      /* Set for CCP (NOT FUNCTIONAL!) */

#ifndef VJ_SUPPORT
#define VJ_SUPPORT                      0      /* Set for VJ header compression. */
#endif

#ifndef MD5_SUPPORT
#define MD5_SUPPORT                     0      /* Set for MD5 (see also CHAP) */
#endif


/*
 * Timeouts.
 */
#define FSM_DEFTIMEOUT                  6       /* Timeout time in seconds */
#define FSM_DEFMAXTERMREQS              2       /* Maximum Terminate-Request transmissions */
#define FSM_DEFMAXCONFREQS              10      /* Maximum Configure-Request transmissions */
#define FSM_DEFMAXNAKLOOPS              5       /* Maximum number of nak loops */

#define UPAP_DEFTIMEOUT                 6       /* Timeout (seconds) for retransmitting req */
#define UPAP_DEFREQTIME                 30      /* Time to wait for auth-req from peer */

#define CHAP_DEFTIMEOUT                 6       /* Timeout time in seconds */
#define CHAP_DEFTRANSMITS               10      /* max # times to send challenge */


/* Interval in seconds between keepalive echo requests, 0 to disable. */
#if 1
#define LCP_ECHOINTERVAL                0
#else
#define LCP_ECHOINTERVAL                10
#endif

/* Number of unanswered echo requests before failure. */
#define LCP_MAXECHOFAILS                3

/* Max Xmit idle time (in jiffies) before resend flag char. */
#define PPP_MAXIDLEFLAG                 100

/*
 * Packet sizes
 *
 * Note - lcp shouldn't be allowed to negotiate stuff outside these
 *    limits.  See lcp.h in the pppd directory.
 * (XXX - these constants should simply be shared by lcp.c instead
 *    of living in lcp.h)
 */
#define PPP_MTU                         1500     /* Default MTU (size of Info field) */
#if 0
#define PPP_MAXMTU  65535 - (PPP_HDRLEN + PPP_FCSLEN)
#else
#define PPP_MAXMTU                      1500 /* Largest MTU we allow */
#endif
#define PPP_MINMTU                      64
#define PPP_MRU                         1500     /* default MRU = max length of info field */
#define PPP_MAXMRU                      1500     /* Largest MRU we allow */
#define PPP_DEFMRU                      296             /* Try for this */
#define PPP_MINMRU                      128             /* No MRUs below this */


#define MAXNAMELEN                      256     /* max length of hostname or name for auth */
#define MAXSECRETLEN                    256     /* max length of password or secret */

#endif /* PPP_SUPPORT */

/* checksum options - set to zero for hardware checksum support */

#define CHECKSUM_GEN_IP                 1
#define CHECKSUM_GEN_UDP                1
#define CHECKSUM_GEN_TCP                1
#define CHECKSUM_CHECK_IP               1
#define CHECKSUM_CHECK_UDP              1
#define CHECKSUM_CHECK_TCP              1

/* Debugging options all default to off */

#ifndef DBG_TYPES_ON
#define DBG_TYPES_ON                    DBG_TRACE
#endif

#ifndef ETHARP_DEBUG
#define ETHARP_DEBUG                    DBG_TRACE
#endif

#ifndef NETIF_DEBUG
#define NETIF_DEBUG                     DBG_TRACE
#endif

#ifndef PBUF_DEBUG
#define PBUF_DEBUG                      DBG_TRACE
#endif

#ifndef API_LIB_DEBUG
#define API_LIB_DEBUG                   DBG_TRACE
#endif

#ifndef API_MSG_DEBUG
#define API_MSG_DEBUG                   DBG_TRACE
#endif

#ifndef SOCKETS_DEBUG
#define SOCKETS_DEBUG                   DBG_TRACE
#endif

#ifndef ICMP_DEBUG
#define ICMP_DEBUG                      DBG_TRACE
#endif

#ifndef INET_DEBUG
#define INET_DEBUG                      DBG_TRACE
#endif

#ifndef IP_DEBUG
#define IP_DEBUG                        DBG_TRACE
#endif

#ifndef IP_REASS_DEBUG
#define IP_REASS_DEBUG                  DBG_TRACE
#endif

#ifndef RAW_DEBUG
#define RAW_DEBUG                       DBG_TRACE
#endif

#ifndef MEM_DEBUG
#define MEM_DEBUG                       DBG_TRACE
#endif

#ifndef MEMP_DEBUG
#define MEMP_DEBUG                      DBG_TRACE
#endif

#ifndef SYS_DEBUG
#define SYS_DEBUG                       DBG_TRACE
#endif

#ifndef TCP_DEBUG
#define TCP_DEBUG                       DBG_TRACE
#endif

#ifndef TCP_INPUT_DEBUG
#define TCP_INPUT_DEBUG                 DBG_TRACE
#endif

#ifndef TCP_FR_DEBUG
#define TCP_FR_DEBUG                    DBG_TRACE
#endif

#ifndef TCP_RTO_DEBUG
#define TCP_RTO_DEBUG                   DBG_TRACE
#endif

#ifndef TCP_REXMIT_DEBUG
#define TCP_REXMIT_DEBUG                DBG_TRACE
#endif

#ifndef TCP_CWND_DEBUG
#define TCP_CWND_DEBUG                  DBG_TRACE
#endif

#ifndef TCP_WND_DEBUG
#define TCP_WND_DEBUG                   DBG_TRACE
#endif

#ifndef TCP_OUTPUT_DEBUG
#define TCP_OUTPUT_DEBUG                DBG_TRACE
#endif

#ifndef TCP_RST_DEBUG
#define TCP_RST_DEBUG                   DBG_TRACE
#endif

#ifndef TCP_QLEN_DEBUG
#define TCP_QLEN_DEBUG                  DBG_TRACE
#endif

#ifndef UDP_DEBUG
#define UDP_DEBUG                       DBG_TRACE
#endif

#ifndef TCPIP_DEBUG
#define TCPIP_DEBUG                     DBG_TRACE
#endif

#ifndef PPP_DEBUG 
#define PPP_DEBUG                       DBG_TRACE
#endif

#ifndef SLIP_DEBUG 
#define SLIP_DEBUG                      DBG_TRACE
#endif

#ifndef DHCP_DEBUG 
#define DHCP_DEBUG                      DBG_TRACE
#endif


#ifndef DBG_MIN_LEVEL
#define DBG_MIN_LEVEL                   DBG_MASK_LEVEL
#endif


#endif /* LWIPOPTS_H */
